﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Management;
using System.Data.EntityClient;
using System.Configuration;
using System.IO;
using System.Data.Objects.DataClasses;
using System.Data.Objects;

namespace FileCatalog.UnitTests
{
    /// <summary>
    /// Summary description for Examples
    /// </summary>
    [TestClass]
    public class Examples
    {
        #region Plumbing

        private TestContext testContextInstance;

        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }


        private string GetSerialNumber(string drive)
        {
            ManagementObject disk = new ManagementObject("win32_logicaldisk.deviceid=\"" + drive + ":\"");
            return disk.Properties["VolumeSerialNumber"].Value.ToString();
        }

        #endregion

        #region Example code

        /// <summary>
        /// Populates the database with some test data
        /// </summary>
        /// <param name="testContext">The test context.</param>
        [ClassInitialize()]
        public static void LoadTestData(TestContext testContext)
        {
            // Get the connection string settings from app.config (or web.config)
            ConnectionStringSettings connStringSetting = 
                ConfigurationManager.ConnectionStrings[Constants.FullStrengthConnectionStringName];

            // Create the context and add some objects
            using (FileCatalogContext context = new FileCatalogContext(connStringSetting.ConnectionString))
            {
                string sourceDirPath = Constants.SourceDirectory;
                
                ReadDirectory(sourceDirPath, context);

                context.SaveChanges();
            }
        }

        private static void ReadDirectory(string path, FileCatalogContext context)
        {
            string driveLetter = path.Substring(0, 1);
            DriveInfo drive = new DriveInfo(driveLetter);

            Volume volume = new Volume
            {
                VolumeId = Guid.NewGuid(),
                Name = drive.Name,
                SerialNumber = "Serial",
                TotalFreeSpace = drive.TotalFreeSpace,
                TotalSize = drive.TotalSize,
                VolumeLabel = drive.VolumeLabel,
                VolumeFormat = drive.DriveFormat
            };

            context.AddToVolume(volume);

            DirectoryInfo dir = new DirectoryInfo(path);

            ReadDirectoryRecursive(volume, context, dir);
        }

        private static void ReadDirectoryRecursive(Volume volume, FileCatalogContext context, DirectoryInfo dir)
        {
            DirectoryItem directoryItem = new DirectoryItem
            {
                FileSystemItemId = Guid.NewGuid(),
                CreationTime = dir.CreationTime,
                Extension = dir.Extension,
                LastAccessTime = dir.LastAccessTime,
                LastWriteTime = dir.LastWriteTime,
                Name = dir.Name
            };

            directoryItem.Volumes.Add(volume);

            context.AddToFileSystemItem(directoryItem);

            var fileItems = from FileInfo file in dir.GetFiles()
                            select new FileItem
                            {
                                FileSystemItemId = Guid.NewGuid(),
                                CreationTime = file.CreationTime,
                                Extension = file.Extension,
                                LastAccessTime = file.LastAccessTime,
                                LastWriteTime = file.LastWriteTime,
                                Length = file.Length,
                                Name = file.Name,
                                Directory = directoryItem,

                            };

            foreach (FileItem file in fileItems)
            {
                file.Volumes.Add(volume);
                context.AddToFileSystemItem(file);
            }

            foreach (DirectoryInfo childDir in dir.GetDirectories())
            {
                ReadDirectoryRecursive(volume, context, childDir);
            }
        }

        [TestMethod]
        [Description("Shows how to retrieve some items with Entity SQL")]
        public void GetFilesWithEntitySQL()
        {
            using (FileCatalogContext context = new FileCatalogContext())
            {
            }
        }

        [TestMethod]
        [Description("Shows how to retrieve some items with include")]
        public void GetFilesWithInclude()
        {
            using (FileCatalogContext context = new FileCatalogContext())
            {
                ObjectQuery<DirectoryItem> directoryQuery = (from DirectoryItem d in context.FileSystemItem.OfType<DirectoryItem>()
                                                         select d) as ObjectQuery<DirectoryItem>;

                // This tells EF to populate child collections
                directoryQuery.Include("Files");
                directoryQuery.Include("ParentDirectory");
                directoryQuery.Include("Volumes");

                foreach (DirectoryItem dir in directoryQuery)
                {
                    Assert.IsNotNull(dir.Files);
                }
            }
        }

        [TestMethod]
        [Description("Shows how to retrieve some items with OfType")]
        public void GetFilesWithOfType()
        {
            using (FileCatalogContext context = new FileCatalogContext())
            {
                // Get directories plus their files
                var directories = context.FileSystemItem.OfType<DirectoryItem>().ToList();

                // We should only have returned directories (not files)
                CollectionAssert.AllItemsAreInstancesOfType(directories, typeof(DirectoryItem));
            }
        }

        //[TestMethod]
        //[Description("Shows how to copy objects between a MDF and SDF")]
        //public void CopyFilesBetweenSQLandCE()
        //{
        //    ConnectionStringSettings sourceConnString = ConfigurationManager.ConnectionStrings[Constants.FullStrengthConnectionStringName];
        //    ConnectionStringSettings destConnString = ConfigurationManager.ConnectionStrings[Constants.LightweightConnectionStringName];

        //    using (FileCatalogContext sourceContext = new FileCatalogContext(sourceConnString.ConnectionString))
        //    using (FileCatalogContext destContext = new FileCatalogContext(destConnString.ConnectionString))
        //    {
        //        ObjectQuery<Volume> volumesInSourceDb = (from Volume v in sourceContext.Volume
        //                                           select v) as ObjectQuery<Volume>;

        //        volumesInSourceDb.MergeOption = MergeOption.NoTracking;

        //        foreach (Volume volume in volumesInSourceDb)
        //        {
        //            destContext.Attach(volume);

        //            destContext.ObjectStateManager.GetObjectStateEntry(volume).SetModified();
        //            //destContext.AddToVolume(volume);
        //        }

        //        destContext.SaveChanges();
        //    }
        //}

     
        #endregion
    }
}
